@@ -46,9 +46,14 @@ class GradescopeClient:
     def verify_logged_in(self):
         if not self.logged_in:
             return False
-        url = self.base_url + "/account"
-        self.last_res = self.session.get(url)
-        return self.last_res.ok
+        # url = self.base_url + "/account"
+        # self.last_res = self.session.get(url)
+        # return self.last_res.ok
+        url = self.base_url + "/login"
+        self.last_res = res = self.session.get(url)
+        # If you are logged in and visit the login page, it returns a 401 error
+        # and will return content b'{"warning":"You must be logged out to access this page."}'
+        return res.status_code == 401
 
     def submit_form(self, url, ref_url, data=None, files=None, header_token=None, json=None):
         headers={
@@ -114,11 +119,11 @@ class GradescopeClient:
             return True
         return False
 
-    def download_scores(self, class_id: str, assignment_id: str) -> bytes:
+    def download_scores(self, class_id: str, assignment_id: str, filetype: str="csv") -> bytes:
         if not self.logged_in:
             print("You must be logged in to download grades!")
             return False
-        self.last_res = self.session.get(f"https://www.gradescope.com/courses/{class_id}/assignments/{assignment_id}/scores.csv")
+        self.last_res = self.session.get(f"https://www.gradescope.com/courses/{class_id}/assignments/{assignment_id}/scores.{filetype}")
         if not self.last_res or not self.last_res.ok:
             print(f"Failed to get a response from gradescope! Got: {self.last_res}")
             return False
@@ -768,7 +773,7 @@ class GradescopeClient:
             if matches:
                 return matches[1]
 
-    def autograder_results(self, class_id: str, assignment_id: str, submission_id: str):
+    def scrape_autograder_results(self, class_id: str, assignment_id: str, submission_id: str):
         """
         Returns a dictionary of test case information:
         {
@@ -843,4 +848,138 @@ class GradescopeClient:
                     "passed": None,
                 })
         return r
-        
\ No newline at end of file
+        
+    def start_export_submissions(self, class_id: str, assignment_id: str) -> int:
+        if not self.logged_in:
+            print("You must be logged in!")
+            return
+        base_url = f"https://www.gradescope.com/courses/{class_id}/assignments/{assignment_id}/"
+        location_url = base_url + f"review_grades"
+        url = base_url + "export"
+        token = self.get_token(location_url, meta="csrf-token")
+        payload = {
+            "authenticity_token": token
+        }
+        self.last_res = res = self.submit_form(url, location_url, data=payload)
+        if res.status_code == 200:
+            return json.loads(res.content).get("generated_file_id")
+        return None
+
+    def check_gon_export_submissions_progress(self, class_id: str, assignment_id: str):
+        if not self.logged_in:
+            print("You must be logged in!")
+            return
+        base_url = f"https://www.gradescope.com/courses/{class_id}/assignments/{assignment_id}/"
+        url = base_url + f"review_grades"
+        self.last_res = self.session.get(url)
+        self.last_soup = BeautifulSoup(self.last_res.content, "html.parser")
+        cdata = GS_CDATA_decoder(soup=self.last_soup)
+        return cdata.get_gon().get("generated_file")
+    
+    def check_export_submissions_progress(self, class_id: str, generated_files_id: str):
+        if not self.logged_in:
+            print("You must be logged in!")
+            return
+        base_url = f"https://www.gradescope.com"
+        location_url = f"/courses/{class_id}"
+        url = base_url + location_url + f"/generated_files/{generated_files_id}.json"
+        self.last_res = self.session.get(url)
+        if self.last_res.ok:
+            return json.loads(self.last_res.content)
+
+    def download_export_submissions(self, class_id: str, generated_files_id: str):
+        if not self.logged_in:
+            print("You must be logged in!")
+            return
+        base_url = f"https://www.gradescope.com"
+        location_url = f"/courses/{class_id}"
+        url = base_url + location_url + f"/generated_files/{generated_files_id}.zip"
+        self.last_res = self.session.get(url)
+        if self.last_res.ok:
+            return self.last_res.content
+
+    def get_assignment_name(self, class_id: str, assignment_id: str) -> str:
+        r = self.grading_get_rubrics(class_id)
+        if r:
+            for a in r.get("assignments", []):
+                if str(a.get("id")) == assignment_id:
+                    return a.get("title")
+
+    def submit_via_repo(self, class_id: str, assignment_id: str, method: str, repo_identifier: str, branch: str, leaderboard_name: str="", owner_id: str=None):
+        """
+        The full branch name MUST be correct
+        Also method must be either github or bitbucket
+        repo_identifier must be the repo id for github or the project name for bitbucket.
+        """
+        if not self.logged_in:
+            print("You must be logged in!")
+            return
+        refurl = self.base_url + f"/courses/{class_id}"
+        url = refurl + f"/assignments/{assignment_id}/submissions"
+        token = self.get_token(refurl, meta="csrf-token")
+        payload = {
+            "utf8": "✓",
+            "authenticity_token": token,
+            "submission[method]": method,
+            "submission[repository]": repo_identifier,
+            "submission[revision]": branch,
+            "submission[leaderboard_name]": leaderboard_name
+        }
+        if owner_id is not None:
+            payload["submission[owner_id]"] = owner_id
+        self.last_res = res = self.submit_form(url, url, data=payload)
+        return res.ok
+            
+
+    def get_bitbucket_branches(self, project_name: str):
+        """
+        E.g. https://www.gradescope.com/bitbucket_projects/ThaumicMekanism/cs61c-lab0_with_git/branches
+        """
+        if not self.logged_in:
+            print("You must be logged in!")
+            return
+        url = f"https://www.gradescope.com/bitbucket_projects/{project_name}/branches"
+        self.last_res = res = self.session.get(url)
+        if res.ok:
+            return json.loads(res.content)
+
+    def get_bitbucket_projects(self):
+        if not self.logged_in:
+            print("You must be logged in!")
+            return
+        url = "https://www.gradescope.com/bitbucket_projects"
+        self.last_res = res = self.session.get(url)
+        if res.ok:
+            return json.loads(res.content)
+
+    def get_github_branches(self, repo_id: str):
+        """
+        """
+        if not self.logged_in:
+            print("You must be logged in!")
+            return
+        url = f"https://www.gradescope.com/github_repositories/{repo_id}/branches"
+        self.last_res = res = self.session.get(url)
+        if res.ok:
+            return json.loads(res.content)
+
+    def get_github_projects(self):
+        if not self.logged_in:
+            print("You must be logged in!")
+            return
+        url = "https://www.gradescope.com/github_repositories"
+        self.last_res = res = self.session.get(url)
+        if res.ok:
+            return json.loads(res.content)
+
+    def get_submission_roster(self, class_id: str, assignment_id: str):
+        if not self.logged_in:
+            print("You must be logged in!")
+            return
+        url = self.base_url + f"/courses/{class_id}/assignments/{assignment_id}/submissions"
+        self.last_res = res = self.session.get(url)
+        if not res.ok:
+            return
+        self.last_soup = soup = BeautifulSoup(res.content, "html.parser")
+        cdata = GS_CDATA_decoder(soup=soup)
+        return cdata.get_gon().get("roster")
\ No newline at end of file
