@@ -14,7 +14,7 @@ from fullGSapi.cli.utils import LoginTokens, course_id_option, assignment_id_opt
 @click.pass_context
 def download_grades(ctx, course: str, assignment: str, filetype: str, output: str):
     """
-    Performs login to Gradescope's APIs.
+    [TA] Downloads the grades for an assignment.
     """
     tokens: LoginTokens = get_tokens(ctx)
     gs = tokens.gsFullapi
